import random
import stddraw


class Board():
    def __init__(self, x, y, difficulty):
        self.x = x
        self.y = y
        self._setupBoard(difficulty)
        
    # difficulty should be a positive number > 1. The higher the number, the easier the game
    def _setupBoard(self, difficulty):
        self.board = [[0 for i in range(self.y)] for j in range(self.x)]
        for i in range(self.x):
            for j in range(self.y):
                r = random.randint(0, difficulty)
                if r == 1:
                    self.board[i][j] = 1 # 1 indicates a mine

    def _tileOutOfBounds(self, x, y):
        return x < 0 or y < 0 or x >= self.x or y >= self.y

    def printBoard(self):
        for i in range(self.x):
            print("_", end='')
        print()

        for j in range(self.y):
            for i in range(self.x):
                print(str(self.board[i][self.y - j - 1]) + "  ", end='')
            print("\n")
        
    def isClear(self, x, y):
        if self._tileOutOfBounds(x, y):
            return True
        return self.board[x][y] == 0

    def isMine(self, x, y):
        if self._tileOutOfBounds(x, y):
            return False
        return self.board[x][y] == 1

    def isChecked(self, x, y):
        if self._tileOutOfBounds(x, y):
            return True
        return self.board[x][y] == 2

    def numOfAdjMines(self, x, y):
        cnt = 0
        for i in range(-1, 2):
            for j in range(-1, 2):
                if  not (i==0 and j==0) and self.isMine(x+i,y+j):
                    cnt += 1
        return cnt

    def checkTile(self, x, y):
        if self.isClear(x, y):
            self.board[x][y] = 2 # 2 indicates a checked tile

    def gameWon(self):
        for i in range(self.x):
            for j in range(self.y):
                if self.board[i][j] == 0:
                    return False
        return True